#include <avr/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <inttypes.h>

#include <stdio.h>
#include "uart0.h"

#include "touchpad.h"
#include "rtc.h"

volatile uint8_t tpAction = TP_NO_ACTION;
volatile uint16_t tpSliderValue = 0;

void tpProcessPacket(uint8_t* buf) {
	static uint8_t mode = TP_NO_ACTION;
	static uint8_t lSector = 0;
	int16_t x, y, z;
	
	// Koordinaten ermitteln
	x = (buf[2]&0x78)<<4;
	x |= buf[0]&0x7F;

	y = (buf[3]&0x78)<<4;
	y |= buf[1]&0x7F;

	z = buf[4];
	
	// Koordinaten interpretieren
	
	// Button 1 (in der Mitte vom Rad)
	if((x > 1300 && x < 1600 && y > 650 && y < 950 && mode == TP_NO_ACTION)
				|| mode == TP_BUTTON1) {
		
		if(z > 0 && mode == TP_NO_ACTION) {
			// Button pressed
			mode = TP_BUTTON1;
			tpAction = TP_ACT_B1_PRESSED;
		} else if(z == 0 && mode == TP_BUTTON1){
			// Button released
			mode = TP_NO_ACTION;
			tpAction = TP_ACT_B1_RELEASED;
		}
					
	// Button 2 (oben links)
	} else if((x > 100 && x < 400 && y > 1100 && y < 1400 && mode == TP_NO_ACTION)
				|| mode == TP_BUTTON2) {
		
		if(z > 0 && mode == TP_NO_ACTION) {
			// Button pressed
			mode = TP_BUTTON2;
			tpAction = TP_ACT_B2_PRESSED;
		} else if(z == 0 && mode == TP_BUTTON2){
			// Button released
			mode = TP_NO_ACTION;
			tpAction = TP_ACT_B2_RELEASED;
		}
			
	// Button 3 (oben mitte-links)
	} else if((x > 100 && x < 400 && y > 800 && y < 1100 && mode == TP_NO_ACTION)
				|| mode == TP_BUTTON3) {
		
		if(z > 0 && mode == TP_NO_ACTION) {
			// Button pressed
			mode = TP_BUTTON3;
			tpAction = TP_ACT_B3_PRESSED;
		} else if(z == 0 && mode == TP_BUTTON3){
			// Button released
			mode = TP_NO_ACTION;
			tpAction = TP_ACT_B3_RELEASED;
		}
						
	// Button 4 (oben mitte-rechts)
	} else if((x > 100 && x < 400 && y > 350 && y < 650 && mode == TP_NO_ACTION)
				|| mode == TP_BUTTON4) {
		
		if(z > 0 && mode == TP_NO_ACTION) {
			// Button pressed
			mode = TP_BUTTON4;
			tpAction = TP_ACT_B4_PRESSED;
		} else if(z == 0 && mode == TP_BUTTON4){
			// Button released
			mode = TP_NO_ACTION;
			tpAction = TP_ACT_B4_RELEASED;
		}
						
	// Button 5 (oben rechts)
	} else if((x > 100 && x < 400 && y > 50 && y < 350 && mode == TP_NO_ACTION)
				|| mode == TP_BUTTON5) {
		
		if(z > 0 && mode == TP_NO_ACTION) {
			// Button pressed
			mode = TP_BUTTON5;
			tpAction = TP_ACT_B5_PRESSED;
		} else if(z == 0 && mode == TP_BUTTON5){
			// Button released
			mode = TP_NO_ACTION;
			tpAction = TP_ACT_B5_RELEASED;
		}
					
	// Slider
	} else if((x > 400 && x < 800 && z > 0 && mode == TP_NO_ACTION)
				|| mode == TP_SLIDER) {
		int16_t value = y-250;

		mode = TP_SLIDER;
		tpAction = TP_ACT_SLIDER_CHANGED;
		
		if(value < 0) value = 0;
		if(value > 1000) value = 1000;
		
		tpSliderValue = 1000-value;
		
	// Drehrad
	} else if((x > 800 && z > 0 && mode == TP_NO_ACTION)
				|| mode == TP_WHEEL) {
					
		int16_t x1 = x-1450;
		int16_t y1 = y-800;
		uint8_t sector;
		
		mode = TP_WHEEL;
		
		// Sektor ermitteln
		// Alternative zu den viele ifs wre den Winkel mit Hilfe von
		// Umkehr-Sinus, und -Cosinus zu bestimmen, das gbe zudem genauere
		// Werte, wrde aber auch einen wesentlich hheren Rechenaufwand bedeuten.
		if(x1 > 0) {
			if(y1 > 0) {
				if(y1 > x1) {
					if(y1 > x1+(x1>>1)) {
						sector = 0;
					} else {
						sector = 1;
					}
				} else {
					if(y1 > (x1>>1)) {
						sector = 2;
					} else {
						sector = 3;
					}
				}
			} else {
				y1 = -y1;
				if(y1 < x1) {
					if(y1 < (x1>>1)) {
						sector = 4;
					} else {
						sector = 5;
					}
				} else {
					if(y1 < x1+(x1>>1)) {
						sector = 6;
					} else {
						sector = 7;
					}
				}
			}
		} else {
			x1 = -x1;
			if(y1 > 0) {
				if(y1 > x1) {
					if(y1 > x1+(x1>>1)) {
						sector = 15;
					} else {
						sector = 14;
					}
				} else {
					if(y1 > (x1>>1)) {
						sector = 13;
					} else {
						sector = 12;
					}
				}
			} else {
				y1 = -y1;
				if(y1 < x1) {
					if(y1 < (x1>>1)) {
						sector = 11;
					} else {
						sector = 10;
					}
				} else {
					if(y1 < x1+(x1>>1)) {
						sector = 9;
					} else {
						sector = 8;
					}
				}
			}
		}
		
		if(lSector != sector) {
			if(sector == 0 && lSector == 15) {
				tpAction = TP_ACT_WHEEL_DOWN;
			} else if(sector == 15 && lSector == 0) {
				tpAction = TP_ACT_WHEEL_UP;
			} else if(sector > lSector) {
				tpAction = TP_ACT_WHEEL_DOWN;
			} else {
				tpAction = TP_ACT_WHEEL_UP;
			}
		}
		lSector = sector;
	}
	
	if(z == 0) {
		mode = TP_NO_ACTION;
	}
}

SIGNAL(TP_UART_RECV_IRQ) {
	// Empfangsbuffer
	static uint8_t packetBuf[5];
	static uint8_t packetIndex = 0;
	
	// zum debuggen
	PORTD |= (1<<PD7);

	// Empfangenes Byte in den Packet Buffer schreiben
	packetBuf[packetIndex++] = TP_UDR;
	
	// Packet vollstndig empfangen?
	if(packetIndex == 5) {
		packetIndex = 0;
		
		// Das letzte Byte muss < 0x80 sein, damit das empfangene Paket gltig ist
		if(packetBuf[4] < 0x80) {
			// Packet analysieren
			tpProcessPacket(packetBuf);
		
		// Das empfangene Packet ist nicht synchron...
		} else {
			// Index um 1 verschieben um Synchronisation zu erreichen. Es knnen
			// maximal 4 Pakete verloren gehen, aber das macht nichts.
			packetIndex++;
		}
	}
	
	// zum debuggen
	PORTD &= ~(1<<PD7);
}

/*
 * Wartet darauf, das ein Byte vom UART empfangen wird.
 * Wird nur beim Init verwendet.
 * berschreitet die Wartezeit [timeout] ms, wird -1 zurckgegeben.
 */
int tpGet(uint16_t timeout) {
	date_t to = rtcTime;
	rtcDateAdd(&to, RTC_SUB_SEC, timeout/(1000/RTC_IRQS_SEC));
	
	// Auf Empfang warten
	while(!(TP_UCSRA & (1<<RXC))) {
		// Bei Timeout -1 returnen
		if(rtcCompToSystime(&to) < 0) return -1;
	}
	
	// Empfangenes Byte returnen
	return TP_UDR;
}

/*
 * Sendet ein Byte bers UART im Polling Mode. Wird nur whrend des Inits
 * verwendet.
 */
void tpPut(uint8_t data) {
	// Auf Platz im UART Register warten
	while(!(TP_UCSRA & (1<<UDRE)));
	// Byte senden
	TP_UDR = data;
}

void tpCommand(uint8_t cmd) {
	
	// Kommando senden
	tpPut(0);
	tpPut(cmd);
	
	// Antwort abwarten
	while(tpGet(100) >= 0);
}

/**
 * Initialisiert das Touchpad im Extended Packet Mode
 */
void touchpadInit(void) {
	// zum debuggen
	DDRD |= (1<<PD7);
	
	// RTS auf Output und Low (Reset)
	TP_RTS_DDR |= (1<<TP_RTS);
	TP_RTS_PORT &= ~(1<<TP_RTS);
	
	// UART initialisieren
	TP_UCSRB = (1<<RXEN)|(1<<TXEN);
	// 1200 Baud
	TP_UBRRL = (F_CPU/(16*1200UL)-1)&0xFF;
	TP_UBRRH = (F_CPU/(16*1200UL)-1)>>8;

	// noch ein bisschen warten
	_delay_us(50);

	// RTS auf High
	TP_RTS_PORT |= (1<<TP_RTS);
	
	// Nach dem Reset sendet das Touchpad ein paar Zeichen, die aber nicht
	// weiter interresieren...
	tpGet(500);
	while(tpGet(50) >= 0);
	
	// Touchpad in den Extended Packet Mode schalten.
	// Die Befehle wurden vom PC-Treiber abgeguckt, ich wei nicht wirklich was
	// sie bedeuten :)
	tpCommand(0x00);
	tpCommand(0xE9);
	tpCommand(0xF5);
	tpCommand(0xF5);
	tpCommand(0xE7);
	tpCommand(0xE6);
	tpCommand(0xE7);
	tpCommand(0xE9);
	tpCommand(0xF4);
	tpCommand(0xF3);
	tpCommand(0x0A);
	tpCommand(0xF3);
	tpCommand(0xC8);
	tpCommand(0xF4);
	tpCommand(0xF4);
	tpCommand(0xF4);
	tpCommand(0xF4);
	tpCommand(0xF3);
	tpCommand(0x0A);
	tpCommand(0xF2);
	tpCommand(0xF3);
	tpCommand(0x0A);
	tpCommand(0xF5);
	
	// jetzt auf 9600 Baud umschalten
	TP_UBRRL = (F_CPU/(16*9600UL)-1)&0xFF;
	TP_UBRRH = (F_CPU/(16*9600UL)-1)>>8;
	
	tpCommand(0xEC);
	tpCommand(0xF4);
	
	// Receive Interrupt aktivieren
	TP_UCSRB |= (1<<RXCIE);
}

/**
 * Gibt die Aktion seit der letzten Abfrage zurck. Wenn keine Aktion
 * vorgefallen ist, wird TP_NO_ACTION zurckgegeben.
 */
uint8_t touchpadGetAction(void) {
	uint8_t action = tpAction;
	tpAction = TP_NO_ACTION;
	return action;
}
