#ifndef UART0_H
#define UART0_H

#include <avr/pgmspace.h>
#include <stdio.h>

//***********
//* defines *
//***********

#define F_OSC					18432000UL

// Buffergren
#define UART0_RX_BUFFER_SIZE	128		// Empfangsbuffer in Byte
#define UART0_TX_BUFFER_SIZE	128		// Sendebuffer in Byte

// Baudrate
#define UART0_BAUDRATE		115200UL

// USART Hardware Register
#define UART0_UCSRA			UCSR0A
#define UART0_UCSRB			UCSR0B
#define UART0_UCSRC			UCSR0C
#define UART0_UBRR			UBRR0L
#define UART0_UDR			UDR0

// Interrupt-Namen
#define UART0_RECV_IRQ		SIG_UART0_RECV		// Receive Complete Interrupt
#define UART0_TRANS_IRQ		SIG_UART0_DATA		// Data Register Empty Interrupt

// Flags
#define UART0_ECHO			5		// Empfangene Zeichen werden geechot
#define UART0_RECV			6		// Flag wird gesetzt, wenn eine vollstndige
									//  Nachricht im Empfangsbuffer liegt.
									//  Wird in extUartParseMsg() gelscht.
#define UART0_BUF_OV		7		// Overflow-Flag


//*********************
//* Gloable Variablen *
//*********************
extern FILE* uart0IO;
extern volatile uint8_t uart0Flags;
extern uint16_t uart0Timeout;


//************************
//* Funktions-Prototypen *
//************************
int  uart0Get(void);				// liest ein Zeichen
int  uart0Put(char c);				// schreibt ein Zeichen
void uart0Init(void);				// initialisiert das UART
void uart0FlushRxBuffer(void);		// Verwirft den RxBuffer
void uart0FlushTxBuffer(void);		// Verwirft den TxBuffer
void uart0Disable(void);			// Deaktiviert das UART

#define uart0Puts(str) {fputs(str, uart0IO);}
#define uart0PutsP(str) {fputs_P(PSTR(str), uart0IO);}
#define uart0SetTimeout(to) {uart0Timeout=to;}

#endif
