/*
 * Copyright:      Fabian Maximilian Thiele  mailto:me@apetech.de
 * Author:         Fabian Maximilian Thiele
 * Remarks:        this Copyright must be included
 * known Problems: none
 * Version:        1.0
 * Description:    avr-gcc Library for Cirque Touchpads with RS232 Interface
 * 
 */

#ifndef TOUCHPAD_H
#define TOUCHPAD_H


//
// Defines
//

// AVR Taktrate
#define F_CPU					18432000

// Receive Interrupt Name
#define TP_UART_RECV_IRQ		SIG_UART1_RECV
// UART Control Register A
#define TP_UCSRA				UCSR1A
// UART Control Register B
#define TP_UCSRB				UCSR1B
// UART Baud Rate Register
#define TP_UBRRL				UBRR1L
#define TP_UBRRH				UBRR1H
// Empfangs Register Name des verwendeten UARTs
#define TP_UDR					UDR1

// Port Definitionen fr RTS (Reset des Touchpads)
#define TP_RTS_PORT				PORTD
#define TP_RTS_DDR				DDRD
#define TP_RTS					PD4

// Input Modes (aktive Schaltflche)
#define TP_BUTTON1				1
#define TP_BUTTON2				2
#define TP_BUTTON3				3
#define TP_BUTTON4				4
#define TP_BUTTON5				5
#define TP_SLIDER				6
#define TP_WHEEL				7

// Actions
#define TP_NO_ACTION			0
#define TP_ACT_B1_PRESSED		1
#define TP_ACT_B1_RELEASED		2
#define TP_ACT_B2_PRESSED		3
#define TP_ACT_B2_RELEASED		4
#define TP_ACT_B3_PRESSED		5
#define TP_ACT_B3_RELEASED		6
#define TP_ACT_B4_PRESSED		7
#define TP_ACT_B4_RELEASED		8
#define TP_ACT_B5_PRESSED		9
#define TP_ACT_B5_RELEASED		10
#define TP_ACT_SLIDER_CHANGED	11
#define TP_ACT_WHEEL_UP			12
#define TP_ACT_WHEEL_DOWN		13


//
// Funktionsprototypen
//

// Touchpad initialisieren
void touchpadInit(void);
// letzte Aktion abfragen
uint8_t touchpadGetAction(void);


//
// Globale Variablen
//

// Enthlt den Wert des Sliders (0 - 1000)
extern volatile uint16_t tpSliderValue;


#endif
